--- old/./Makefile	2013-10-21 19:01:50.847494139 +0000
+++ new/./Makefile	2013-10-21 19:01:50.860997208 +0000
@@ -74,6 +74,7 @@
 	libxml \
 	libz \
 	make \
+	node.js \
 	nss-nspr \
 	openssl1x \
 	perl
@@ -128,7 +129,7 @@
 	(cd gcc4 && \
 	    PKG_CONFIG_LIBDIR="" \
 	    STRAP=$(STRAP) \
-	    $(MAKE) DESTDIR=$(DESTDIR) install)
+	    $(MAKE) DESTDIR=$(DESTDIR) install strapfix)
 
 $(SUBDIRS): $(DESTDIR)/usr/bin/gcc
 	(cd $@ && \
--- old/gcc4/Makefile	2013-10-21 19:01:51.137919875 +0000
+++ new/gcc4/Makefile	2013-10-21 19:01:51.151957765 +0000
@@ -54,6 +54,24 @@
 	CFLAGS_FOR_TARGET="$(CFLAGS)"
 
 #
+# The runtime libraries that we build into proto.strap should be made to search
+# for their dependencies there as well, so that programs built to run on the
+# build machine (from proto.strap) find the correct copies of these libraries.
+#
+ifeq ($(STRAP),strap)
+	STRAPFIX_LINKS= \
+		$(DESTDIR)/lib/64 \
+		$(DESTDIR)/usr/lib/64
+	STRAPFIX_FILES_32 = \
+		$(DESTDIR)/usr/lib/libgcc_s.so.1 \
+		$(DESTDIR)/usr/lib/libstdc++.so.6
+	STRAPFIX_FILES_64 = \
+		$(DESTDIR)/usr/lib/64/libgcc_s.so.1 \
+		$(DESTDIR)/usr/lib/64/libstdc++.so.6
+	STRAPFIX_FILES = $(STRAPFIX_FILES_32) $(STRAPFIX_FILES_64)
+endif
+
+#
 # Unlike everything else, gcc is built to be a cross-compiler, really.  It
 # never runs on the target system, only the build system.  So it should not
 # be using the proto area's headers or libraries.  Its own libraries will be
@@ -108,3 +126,19 @@
 	-rmdir .unpack32
 	chmod 755 $@/configure
 	touch $@/configure
+
+.PHONY: strapfix
+strapfix: $(STRAPFIX_LINKS) $(STRAPFIX_FILES) | install
+	for f in $(STRAPFIX_FILES_32); do \
+		/usr/bin/elfedit -e \
+		    'dyn:runpath $(DESTDIR)/lib:$(DESTDIR)/usr/lib' \
+		    $$f; \
+	done
+	for f in $(STRAPFIX_FILES_64); do \
+		/usr/bin/elfedit -e \
+		    'dyn:runpath $(DESTDIR)/lib/64:$(DESTDIR)/usr/lib/64' \
+		    $$f; \
+	done
+
+$(STRAPFIX_LINKS):
+	ln -s amd64 $@
--- old/libz/Makefile	2013-10-21 19:01:51.416999556 +0000
+++ new/libz/Makefile	2013-10-21 19:01:51.430308223 +0000
@@ -63,6 +63,11 @@
 LDSHARED_FLAGS.64 =	$(LDSHARED_FLAGS) $(SYSLIBDIRS:%=-L$(DESTDIR)/%/64)
 LDSHARED_LIBS =		-lc
 
+ifeq ($(STRAP),strap)
+	LDSHARED_FLAGS.32 += $(SYSLIBDIRS:%=-R$(DESTDIR)/%)
+	LDSHARED_FLAGS.64 += $(SYSLIBDIRS:%=-R$(DESTDIR)/%/64)
+endif
+
 LDFLAGS +=	-L. -lz -lc
 LDFLAGS.64 +=	-L. -lz -lc
 
--- old/node.js/Makefile	2013-10-21 19:01:51.684265063 +0000
+++ new/node.js/Makefile	2013-10-21 19:01:51.697722315 +0000
@@ -22,7 +22,7 @@
 #
 
 MAJOR_VER =	0.8
-VER =	node-v0.8.20
+VER =		node-v0.8.20
 
 include ../Makefile.defs
 
@@ -46,6 +46,10 @@
 #
 LDFLAGS +=	-lumem
 
+ifeq ($(STRAP),strap)
+	LDFLAGS += $(SYSLIBDIRS:%=-R$(DESTDIR)/%)
+endif
+
 AUTOCONF_OPTS += \
 	--with-dtrace \
 	--openssl-use-sys \
--- old/openssl1x/Makefile	2013-10-21 19:01:51.960518027 +0000
+++ new/openssl1x/Makefile	2013-10-21 19:01:51.976086579 +0000
@@ -83,6 +83,11 @@
 
 LIBS +=	-lsocket -lnsl
 
+ifeq ($(STRAP),strap)
+	LDFLAGS +=	$(SYSLIBDIRS:%=-R$(DESTDIR)/%)
+	LDFLAGS.64 +=	$(SYSLIBDIRS:%=-R$(DESTDIR)/%/64)
+endif
+
 PATCHES =	Patches/*
 
 AUTOCONF_ENV =
